/* 
 * winCloser Gnome-panel applet
 *
 * Author: Jochen Baier, email@jochen-baier.de
 * License: GPL
 * 
 */


#include <gnome.h>
#include <panel-applet.h>
#include <panel-applet-gconf.h>
#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1  
#include <libwnck/libwnck.h>

#include "config.h"
#include "common.h"
#include "window.h"
#include "button-widget.h"
#include "single-close.h"
 
gboolean debug=FALSE;

static void
applet_back_change (PanelApplet *a, PanelAppletBackgroundType	type,
  GdkColor *color, GdkPixmap			*pixmap, Applet			*applet)
{

  GtkRcStyle *rc_style;
  GtkStyle *style;

  gtk_widget_set_style (GTK_WIDGET (applet->panel_applet), NULL);
  rc_style = gtk_rc_style_new ();
  gtk_widget_modify_style (GTK_WIDGET (applet->panel_applet), rc_style);
  gtk_rc_style_unref (rc_style);

  switch (type) {
    case PANEL_COLOR_BACKGROUND:
      gtk_widget_modify_bg (GTK_WIDGET (applet->panel_applet),
        GTK_STATE_NORMAL, color);

    break;

    case PANEL_PIXMAP_BACKGROUND:
      style = gtk_style_copy (GTK_WIDGET (
      applet->panel_applet)->style);
      if (style->bg_pixmap[GTK_STATE_NORMAL])
        g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
      style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref(pixmap);
      gtk_widget_set_style (GTK_WIDGET (applet->panel_applet), style);
      g_object_unref (style);

    break;

    case PANEL_NO_BACKGROUND:
    default:
    break;
  }

}

static Applet *
create_applet (PanelApplet *panel_applet)
{ 


  Applet *applet = g_new0 (Applet, 1);   
  applet->panel_applet = panel_applet;

  applet->button=button_widget_new ();
  gtk_container_add (GTK_CONTAINER (applet->panel_applet), applet->button);
  gtk_widget_show (applet->button);

  GtkTooltips *tips=gtk_tooltips_new();
  gtk_tooltips_set_tip (tips, applet->button, 
      "Click to close all windows\non current workspace", NULL);
  gtk_tooltips_enable(tips);

  return applet; 

}

static void
about_cb (BonoboUIComponent *uic, Applet *applet,
  const gchar *verbname)
{
  static const gchar *authors [] = {
  "Jochen Baier <email@jochen-baier.de>\n", NULL};

  gtk_show_about_dialog (NULL,
  "name",		_("winCloser"),
  "version",	VERSION,
  "comments",	("Close all windows on current workspace."),
  "copyright",	"\xC2\xA9 2007 Jochen Baier",
  "authors",	authors,
  "logo-icon-name",	"wincloser32",
  NULL);
} 

static void
kill_single_cb (BonoboUIComponent *uic, Applet *applet,
  const gchar *verbname)
{  
  panel_force_quit (gdk_screen_get_default());
}

static void
kill_cb (BonoboUIComponent *uic, Applet *applet,
  const gchar *verbname)
{ 

  close_current_workspace (applet, TRUE);
}

static void
killall_cb (BonoboUIComponent *uic, Applet *applet,
  const gchar *verbname)
{ 

  close_all_workspace (applet);
} 


static void
destroy_cb (GtkObject *object, Applet *applet)
{

  if (debug) printf ("destroy_cb\n");
 
  gtk_widget_destroy (GTK_WIDGET (applet->panel_applet));
  g_free (applet);

}

gboolean 
button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  Applet *applet = (Applet*) user_data;

  if (event->button != 1)
    return FALSE;

  if (debug) printf ("clicked\n");
 
  close_current_workspace (applet, FALSE);

  return FALSE;
}

static const char menu_xml [] =
   "<popup name=\"button3\">\n"
   "<menuitem name=\"Kill all windows\" "
   "           verb=\"Kill\" "
   "           _label=\"_Kill all windows\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gtk-cancel\"/>\n"
   "<menuitem name=\"Kill a single window\" "
   "           verb=\"KillSingle\" "
   "           _label=\"Kill a _single window\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gtk-add\"/>\n" 
   "<separator/>\n" 
   "<menuitem name=\"Close windows on all workspaces\" "
   "           verb=\"Close\" "
   "           _label=\"_Close windows on all workspaces\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gtk-close\"/>\n"
   "<separator/>\n"
   "   <menuitem name=\"About Item\" "
   "           verb=\"About\" "
   "           _label=\"_About\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gnome-stock-about\"/>\n"
   "</popup>\n";

static const BonoboUIVerb menu_verbs [] = {
        BONOBO_UI_UNSAFE_VERB ("Kill", kill_cb),
        BONOBO_UI_UNSAFE_VERB ("KillSingle", kill_single_cb),
        BONOBO_UI_UNSAFE_VERB ("Close", killall_cb), 
        BONOBO_UI_UNSAFE_VERB ("About", about_cb),
        BONOBO_UI_VERB_END
}; 

static gboolean
applet_init(PanelApplet *panel_applet)
{
  Applet *applet;
  
  gtk_window_set_default_icon_name ("wincloser32");
  applet = create_applet (panel_applet);
  panel_applet_set_flags (applet->panel_applet, PANEL_APPLET_EXPAND_MINOR);
  panel_applet_setup_menu (applet->panel_applet, menu_xml,
    menu_verbs, applet);
  
  g_signal_connect (applet->panel_applet, "change_background",
    G_CALLBACK (applet_back_change), applet);

  g_signal_connect (GTK_WIDGET(applet->panel_applet), "destroy",
    G_CALLBACK (destroy_cb), applet);

  g_signal_connect ((gpointer) applet->button, "button_release_event",
                    G_CALLBACK (button_release_event),
                    (gpointer) applet);

  gtk_widget_show (GTK_WIDGET (applet->panel_applet));

  return TRUE;
}

static gboolean
wincloser_applet_factory (PanelApplet *panel_applet, const gchar *iid, gpointer     data)
{

  gboolean retval = FALSE;

  if (!strcmp (iid, "OAFIID:GNOME_wincloserApplet"))
    retval = applet_init (panel_applet); 
  
  if (retval == FALSE) {
    exit (-1);
  }
  
  return retval;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_wincloserApplet_Factory",
  PANEL_TYPE_APPLET, "wincloser", "0", wincloser_applet_factory, NULL)
